unit fNavegador;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.OleCtrls, SHDocVw,
  Vcl.ExtCtrls, cefvcl, ceflib, Vcl.ComCtrls, uJsonMaps,
  uGoogleMaps;

type


  TfrmNavegador = class(TForm)
    Panel1: TPanel;
    crm: TChromium;
    memPontos: TMemo;
    Panel2: TPanel;
    btnLocalizarPontos: TButton;
    btnAtualizar: TButton;
    btnCalcularRota: TButton;
    btnInfo: TButton;
    StatusBar: TStatusBar;
    Panel3: TPanel;
    memRetorno: TMemo;
    memJson: TMemo;
    Splitter1: TSplitter;
    Splitter2: TSplitter;
    lbxInstrucoes: TListBox;
    Splitter3: TSplitter;
    pnlInstrucoes: TPanel;
    Label1: TLabel;
    procedure btnLocalizarPontosClick(Sender: TObject);
    procedure btnAtualizarClick(Sender: TObject);
    procedure btnCalcularRotaClick(Sender: TObject);
    procedure crmProcessMessageReceived(Sender: TObject;
      const browser: ICefBrowser; sourceProcess: TCefProcessId;
      const message: ICefProcessMessage; out Result: Boolean);
    procedure btnInfoClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  private
    procedure LocalizarPontos;
    procedure TracarRota;
    procedure MonitorarEventoDeAlteracaoDeRota;
    procedure CarregarMapa;
    { Private declarations }
  public
    { Public declarations }
  end;

  TCustomRenderProcessHandler = class(TCefRenderProcessHandlerOwn)
  protected
    function OnProcessMessageReceived(const browser: ICefBrowser;
      sourceProcess: TCefProcessId; const message: ICefProcessMessage): Boolean;
      override;
  end;

var
  frmNavegador: TfrmNavegador;

implementation

{$R *.dfm}

procedure TfrmNavegador.btnLocalizarPontosClick(Sender: TObject);
begin
  LocalizarPontos;
end;

procedure TfrmNavegador.crmProcessMessageReceived(Sender: TObject;
  const browser: ICefBrowser; sourceProcess: TCefProcessId;
  const message: ICefProcessMessage; out Result: Boolean);
begin
  if (message.Name = 'click') then
  begin
    memRetorno.Text := message.ArgumentList.GetString(0);

    Result := True;
  end else begin
    Result := False;
  end;
end;

procedure TfrmNavegador.LocalizarPontos;
var
  li: Integer;
begin
  if crm.Browser <> nil then
  begin
    for li := 0 to memPontos.Lines.Count do
    begin
      crm.Browser.MainFrame.ExecuteJavaScript(
        'LocalizarEndereco('+QuotedStr(memPontos.Lines[li]) + ');',
        'about:blank', 0);
    end;
  end;
end;

procedure TfrmNavegador.TracarRota;
begin
  if crm.Browser <> nil then
  begin
    crm.Browser.MainFrame.ExecuteJavaScript(
      'CalcularRota();',
      'about:blank', 0);
  end;
end;

procedure TfrmNavegador.MonitorarEventoDeAlteracaoDeRota;
begin
  crm.browser.SendProcessMessage(PID_RENDERER, TCefProcessMessageRef.New('visitdom'));
end;

procedure TfrmNavegador.CarregarMapa;
var
  lCaminho : string;
begin
  lCaminho := ExtractFilePath(Application.ExeName);
  crm.Browser.MainFrame.LoadUrl(lCaminho + 'Mapa.htm');
end;

procedure TfrmNavegador.btnCalcularRotaClick(Sender: TObject);
begin
  TracarRota;
end;

procedure TfrmNavegador.btnInfoClick(Sender: TObject);
var
  lJsonMaps : TJsonMaps;
  li: Integer;
  lDirectionsLeg: TDirectionsLeg;
  lDirectionsStep: TDirectionsStep;
  lInstrucoes: string;
begin
  lJsonMaps := TJsonMaps.Create;
  memJson.Text := lJsonMaps.CaregarJson(memRetorno.Text);
  lbxInstrucoes.Clear;
  lJsonMaps.MontarInformacoesDeRota(memRetorno.Text);
  for lDirectionsLeg in lJsonMaps.DirectionsRoute.Legs do
  begin
    for lDirectionsStep in lDirectionsLeg.Steps do
    begin
      lInstrucoes := StringReplace(lDirectionsStep.Instructions,'<b>','',[rfReplaceAll,rfIgnoreCase]);
      lInstrucoes := StringReplace(lInstrucoes,'</b>','',[rfReplaceAll,rfIgnoreCase]);
      lbxInstrucoes.Items.Add(lInstrucoes);
    end;
  end;
end;

procedure TfrmNavegador.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  Halt;
end;

procedure TfrmNavegador.FormCreate(Sender: TObject);
begin
  crm.DefaultUrl := ExtractFilePath(Application.ExeName)+'Mapa.htm';
end;

procedure TfrmNavegador.FormShow(Sender: TObject);
begin
  CarregarMapa;
  MonitorarEventoDeAlteracaoDeRota;
end;

procedure TfrmNavegador.btnAtualizarClick(Sender: TObject);
begin
  CarregarMapa;
end;


{ TCustomRenderProcessHandler }

function getpath(const n: ICefDomNode): string;
begin
  Result := '<' + n.Name + '>';
  if (n.Parent <> nil) then
  begin
    //Result := getpath(n.Parent) + Result;
    if (n.Parent.Parent <> nil) then
    begin
      Result := getpath(n.Parent.Parent) + Result;
    end;
  end;
end;

function TCustomRenderProcessHandler.OnProcessMessageReceived(
  const browser: ICefBrowser; sourceProcess: TCefProcessId;
  const message: ICefProcessMessage): Boolean;
var
  lCefStringMap : ICefStringMap;
  lResult : string;
begin
  if (message.Name = 'visitdom') then
    begin
      browser.MainFrame.VisitDomProc(
        procedure(const doc: ICefDomDocument) begin
          doc.GetElementById('edtRotas').AddEventListenerProc('click', True,
            procedure (const event: ICefDomEvent)
            var
              msg: ICefProcessMessage;
              li: Integer;
            begin
              msg := TCefProcessMessageRef.New('click');
              if event.Document.GetElementById('edtRotas').HasElementAttributes then
              begin
                lCefStringMap := TCefStringMapOwn.Create;
                lResult := event.Document.GetElementById('edtRotas').GetElementAttribute('value');
                msg.ArgumentList.SetString(0, lResult);
              end;
              browser.SendProcessMessage(PID_BROWSER, msg);
            end)

        end);
        Result := True;
    end
  else
  Result := False;
end;


initialization
  CefRenderProcessHandler := TCustomRenderProcessHandler.Create;

end.
